# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2012-2024, by Samuel Williams.

require_relative "xrb/version"
require_relative "xrb/native"
require_relative "xrb/builder"
require_relative "xrb/template"

require_relative "xrb/reference"
